package com.mints.goodmoney.ad.express

import android.os.Handler
import android.os.HandlerThread
import android.text.TextUtils
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.*
import com.bumptech.glide.Glide
import com.bytedance.msdk.api.*
import com.bytedance.msdk.api.nativeAd.*
import com.mints.goodmoney.MintsApplication
import com.mints.goodmoney.R
import com.mints.goodmoney.common.Constant
import com.mints.goodmoney.manager.CsjGroMoreManager
import com.mints.goodmoney.manager.TrackManager
import com.mints.goodmoney.ui.widgets.seekbar.BubbleUtils
import com.mints.goodmoney.utils.LogUtil
import com.mints.goodmoney.utils.UIUtils
import com.mints.goodmoney.utils.VideoOptionUtil
import java.util.ArrayList
import java.util.HashMap

private val TAG = GroMoreCarrierExpressManager::class.java.simpleName

class GroMoreCarrierExpressManager : TTSettingConfigCallback {

    companion object {
        val instance: GroMoreCarrierExpressManager by lazy(mode = LazyThreadSafetyMode.SYNCHRONIZED) {
            GroMoreCarrierExpressManager()
        }
    }

    private var mAdStyle = 0
    private var mExpressType = 0

    private var frameLayout: FrameLayout? = null

    private var mHandlerThread: HandlerThread? = null
    private var mWorkHandler: Handler? = null

    private var mTTAdNative: TTUnifiedNativeAd? = null

    init {
        mAdStyle = AdSlot.TYPE_EXPRESS_AD
    }

    fun loadADFrameLayout() {
        // 销毁及移除事件
        destroy()
        frameLayout = FrameLayout(MintsApplication.getContext())

        if (TTMediationAdSdk.configLoadSuccess()) {
            loadAd()
        } else {
            TTMediationAdSdk.registerConfigCallback(this)
        }
    }

    /**
     * config成功，加载信息流
     */
    override fun configLoad() {
        loadAd()
    }

    private fun loadAd() {
        mTTAdNative = TTUnifiedNativeAd(MintsApplication.getContext(), CsjGroMoreManager.AD_UNIT_EXPRESS_ID)

        //视频声音控制设置
        var videoOption: TTVideoOption = VideoOptionUtil.getTTVideoOption()

        //判断是否需要使用GDT Feed 模板渲染2.0；注意！！！！需要向广点通商务申请模渲染2.0权限
        // 注意不设置默认使用 GDT Feed 模板渲染1.0,默认情况下无需执行下方代码
        if (mAdStyle == AdSlot.TYPE_EXPRESS_AD && mExpressType == GDTExtraOption.FeedExpressType.FEED_EXPRESS_TYPE_2) {
            videoOption = VideoOptionUtil.getTTVideoOption2()
        }

        //针对Admob Native的特殊配置项
        val admobNativeAdOptions = AdmobNativeAdOptions()
        admobNativeAdOptions.setAdChoicesPlacement(AdmobNativeAdOptions.ADCHOICES_TOP_RIGHT) //设置广告小标默认情况下，广告选择叠加层会显示在右上角。
                .setRequestMultipleImages(true).isReturnUrlsForImageAssets = true //设置为true，SDK会仅提供Uri字段的值，允许自行决定是否下载实际图片，同时不会提供Drawable字段的值


        // 针对Gdt Native自渲染广告，可以自定义gdt logo的布局参数。该参数可选,非必须。
        val gdtNativeAdLogUtiloParams = FrameLayout.LayoutParams(
                BubbleUtils.dp2px(40),
                BubbleUtils.dp2px(14),
                Gravity.END or Gravity.TOP) // 例如，放在右上角

        /**
         * 创建feed广告请求类型参数AdSlot,具体参数含义参考文档
         * 备注
         * 1: 如果是信息流自渲染广告，设置广告图片期望的图片宽高 ，不能为0
         * 2:如果是信息流模板广告，宽度设置为希望的宽度，高度设置为0(0为高度选择自适应参数)
         */
        /**
         * 创建feed广告请求类型参数AdSlot,具体参数含义参考文档
         * 备注
         * 1: 如果是信息流自渲染广告，设置广告图片期望的图片宽高 ，不能为0
         * 2:如果是信息流模板广告，宽度设置为希望的宽度，高度设置为0(0为高度选择自适应参数)
         */
        val adSlot = AdSlot.Builder()
                .setTTVideoOption(videoOption) //视频声音相关的配置
                .setAdmobNativeAdOptions(admobNativeAdOptions)
                .setAdStyleType(mAdStyle) //必传，表示请求的模板广告还是原生广告，AdSlot.TYPE_EXPRESS_AD：模板广告 ； AdSlot.TYPE_NATIVE_AD：原生广告
                .setImageAdSize(340, 0) // 必选参数 单位dp ，详情见上面备注解释
                .setAdCount(1) //请求广告数量为1到3条
                .setGdtNativeAdLogoParams(gdtNativeAdLogUtiloParams) // 设置gdt logo布局参数。
                .build()

        //请求广告，调用feed广告异步请求接口，加载到广告后，拿到广告素材自定义渲染
        /**
         * 注：每次加载信息流广告的时候需要新建一个TTUnifiedNativeAd，否则可能会出现广告填充问题
         * (例如：mTTAdNative = new TTUnifiedNativeAd(this, mAdUnitId);）
         */
        mTTAdNative?.loadAd(adSlot, object : TTNativeAdLoadCallback {
            override fun onAdLoadedFial(adError: AdError?) {
                LogUtil.e(TAG, "load feed ad error : " + adError?.code + ", " + adError?.message)

            }

            override fun onAdLoaded(ads: MutableList<TTNativeAd>?) {
                if (ads == null || ads.isEmpty()) {
                    LogUtil.e(TAG, "on FeedAdLoaded: ad is null!")
                    return
                }

                for (ttNativeAd in ads) {
                    LogUtil.e(TAG, "   ")
                    LogUtil.e(TAG, "adNetworkPlatformId: " + ttNativeAd.adNetworkPlatformId + "   adNetworkRitId：" + ttNativeAd.adNetworkRitId + "   preEcpm: " + ttNativeAd.preEcpm)
                }

                if (ads.size > 0) {
                    frameLayout?.addView(createAdView(ads[0]))

                    val ttNativeAd = ads[0]
                    val vo = HashMap<String, Any>()
                    vo["adcode"] = ttNativeAd.adNetworkRitId
                    vo["ecpm"] = ttNativeAd.preEcpm
                    vo["adSource"] = ttNativeAd.adNetworkPlatformId
                    vo["adType"] = Constant.GRO_MORE_ADTYPE1
                    TrackManager.getInstance().cmtGroMoreInfo(vo)
                }
            }

        })
    }

    fun getADFrameLayout(): FrameLayout? {
        return frameLayout
    }

    /**
     * 销毁
     */
    private fun destroy() {
        mHandlerThread?.quit()
        mHandlerThread = null
        mWorkHandler?.removeCallbacksAndMessages(null)
        mWorkHandler = null
        TTMediationAdSdk.unregisterConfigCallback(this)

    }

    private fun createAdView(ad: TTNativeAd?): View? {

        if (ad != null && ad.isExpressAd) {
            return getExpressAdView(ad)
        }

        return when {
            ad == null -> {
                return null
            }
            ad.adImageMode == TTAdConstant.IMAGE_MODE_SMALL_IMG -> {
                return getSmallAdView(ad)
            }
            ad.adImageMode == TTAdConstant.IMAGE_MODE_LARGE_IMG -> {
                return getLargeAdView(ad)
            }
            ad.adImageMode == TTAdConstant.IMAGE_MODE_GROUP_IMG -> {
                return getGroupAdView(ad)
            }
            ad.adImageMode == TTAdConstant.IMAGE_MODE_VIDEO -> {
                return getVideoView(ad)
            }
            ad.adImageMode == TTAdConstant.IMAGE_MODE_VERTICAL_IMG -> {
                return getVerticalAdView(ad)
            }
            ad.adImageMode == TTAdConstant.IMAGE_MODE_VIDEO_VERTICAL -> {
                return getVideoView(ad)
            }
            else -> {
                return null
            }
        }
    }

    //渲染模板广告
    private fun getExpressAdView(ad: TTNativeAd): View? {
        val adViewHolder: ExpressAdViewHolder
        val convertView = LayoutInflater.from(MintsApplication.getContext()).inflate(R.layout.listitem_ad_native_express, null, false)
        try {
            adViewHolder = ExpressAdViewHolder()
            adViewHolder.mAdContainerView = convertView.findViewById(R.id.iv_listitem_express) as FrameLayout
            convertView.tag = adViewHolder

            //判断是否存在dislike按钮
//            if (ad.hasDislike()) {
//                ad.setDislikeCallback(MintsApplication.getContext() as Activity?, object : TTDislikeCallback {
//                    override fun onSelected(position: Int, value: String) {
//                        //用户选择不喜欢原因后，移除广告展示
//                    }
//
//                    override fun onCancel() {
//                        LogUtil.d(TAG, "dislike 点击了取消")
//                    }
//
//                    /**
//                     * 拒绝再次提交
//                     */
//                    override fun onRefuse() {}
//                    override fun onShow() {}
//                })
//            }

            //设置点击展示回调监听
            ad.setTTNativeAdListener(object : TTNativeExpressAdListener {
                override fun onAdClick() {
                    LogUtil.d(TAG, "onAdClick")
                }

                override fun onAdShow() {
                    LogUtil.d(TAG, "onAdShow")
                }

                override fun onRenderFail(view: View, msg: String, code: Int) {
                    LogUtil.d(TAG, "onRenderFail   code=$code,msg=$msg")
                }

                // ** 注意点 ** 不要在广告加载成功回调里进行广告view展示，要在onRenderSucces进行广告view展示，否则会导致广告无法展示。
                // 另外，不要直接使用参数view，而是通过ad.getExpressView()来获取广告view。
                override fun onRenderSuccess(view: View, width: Float, height: Float) {
                    LogUtil.d(TAG, "onRenderSuccess")
                    //回调渲染成功后将模板布局添加的父View中
                    if (adViewHolder.mAdContainerView != null) {
                        //获取视频播放view,该view SDK内部渲染，在媒体平台可配置视频是否自动播放等设置。
                        val sWidth: Int
                        val sHeight: Int
                        val video = ad.expressView // 使用该方式获取广告view，而不是直接使用onRenderSuccess的参数view。
                        if (width == TTAdSize.FULL_WIDTH.toFloat() && height == TTAdSize.AUTO_HEIGHT.toFloat()) {
                            sWidth = FrameLayout.LayoutParams.MATCH_PARENT
                            sHeight = FrameLayout.LayoutParams.MATCH_PARENT
                        } else {
                            sWidth = UIUtils.getScreenWidth(MintsApplication.getContext())
                            sHeight = (sWidth * height / width).toInt()
                        }
                        if (video != null) {
                            if (video.parent == null) {
                                val layoutParams = FrameLayout.LayoutParams(sWidth, sHeight)
                                adViewHolder.mAdContainerView!!.removeAllViews()
                                adViewHolder.mAdContainerView!!.addView(video, layoutParams)
                            }
                        }
                    }
                }
            })


            //视频广告设置播放状态回调（可选）
            ad.setTTVideoListener(object : TTVideoListener {
                override fun onVideoStart() {
                    LogUtil.d(TAG, "onVideoStart")
                }

                override fun onVideoPause() {
                    LogUtil.d(TAG, "onVideoPause")
                }

                override fun onVideoResume() {
                    LogUtil.d(TAG, "onVideoResume")
                }

                override fun onVideoCompleted() {
                    LogUtil.d(TAG, "onVideoCompleted")
                }

                override fun onVideoError(adError: AdError) {
                    LogUtil.d(TAG, "onVideoError")
                }
            })
            ad.render()
        } catch (e: Exception) {
            e.printStackTrace()
        }
        return convertView
    }

    /**
     * @param ad
     * @return
     */
    private fun getVerticalAdView(ad: TTNativeAd): View? {
        val convertView = LayoutInflater.from(MintsApplication.getContext()).inflate(R.layout.listitem_ad_vertical_pic, null, false)
        val adViewHolder = VerticalAdViewHolder()
        adViewHolder.mTitle = convertView!!.findViewById(R.id.tv_listitem_ad_title) as TextView
        adViewHolder.mSource = convertView.findViewById(R.id.tv_listitem_ad_source) as TextView
        adViewHolder.mDescription = convertView.findViewById<View>(R.id.tv_listitem_ad_desc) as TextView
        adViewHolder.mVerticalImage = convertView.findViewById(R.id.iv_listitem_image)
        adViewHolder.mIcon = convertView.findViewById<View>(R.id.iv_listitem_icon) as ImageView
        adViewHolder.mDislike = convertView.findViewById<View>(R.id.iv_listitem_dislike) as ImageView
        adViewHolder.mCreativeButton = convertView.findViewById<View>(R.id.btn_listitem_creative) as Button
        adViewHolder.mLogo = convertView.findViewById(R.id.tt_ad_logo) //logoView 建议传入GroupView类型
        val viewBinder: TTViewBinder = TTViewBinder.Builder(R.layout.listitem_ad_vertical_pic)
                .titleId(R.id.tv_listitem_ad_title)
                .decriptionTextId(R.id.tv_listitem_ad_desc)
                .mainImageId(R.id.iv_listitem_image)
                .iconImageId(R.id.iv_listitem_icon)
                .callToActionId(R.id.btn_listitem_creative)
                .sourceId(R.id.tv_listitem_ad_source)
                .logoLayoutId(R.id.tt_ad_logo) //logoView 建议传入GroupView类型
                .build()
        adViewHolder.viewBinder = viewBinder
        convertView.tag = adViewHolder
        bindData(convertView, adViewHolder, ad, viewBinder)
        if (ad.imageUrl != null) {
            adViewHolder.mVerticalImage?.let { Glide.with(MintsApplication.getContext()).load(ad.imageUrl).into(it) }
        }
        return convertView
    }

    //渲染视频广告，以视频广告为例，以下说明
    private fun getVideoView(ad: TTNativeAd): View? {
        val adViewHolder: VideoAdViewHolder
        val viewBinder: TTViewBinder
        val convertView = LayoutInflater.from(MintsApplication.getContext()).inflate(R.layout.listitem_ad_large_video, null, false)
        try {
            adViewHolder = VideoAdViewHolder()
            adViewHolder.mTitle = convertView!!.findViewById<View>(R.id.tv_listitem_ad_title) as TextView
            adViewHolder.mDescription = convertView.findViewById<View>(R.id.tv_listitem_ad_desc) as TextView
            adViewHolder.mSource = convertView.findViewById<View>(R.id.tv_listitem_ad_source) as TextView
            adViewHolder.videoView = convertView.findViewById<View>(R.id.iv_listitem_video) as FrameLayout
            adViewHolder.mIcon = convertView.findViewById<View>(R.id.iv_listitem_icon) as ImageView
            adViewHolder.mDislike = convertView.findViewById<View>(R.id.iv_listitem_dislike) as ImageView
            adViewHolder.mCreativeButton = convertView.findViewById<View>(R.id.btn_listitem_creative) as Button
            adViewHolder.mLogo = convertView.findViewById(R.id.tt_ad_logo) //logoView 建议传入GroupView类型

            //TTViewBinder 是必须类,需要开发者在确定好View之后把Id设置给TTViewBinder类，并在注册事件时传递给SDK
            viewBinder = TTViewBinder.Builder(R.layout.listitem_ad_large_video)
                    .titleId(R.id.tv_listitem_ad_title)
                    .sourceId(R.id.tv_listitem_ad_source)
                    .decriptionTextId(R.id.tv_listitem_ad_desc)
                    .mediaViewIdId(R.id.iv_listitem_video)
                    .callToActionId(R.id.btn_listitem_creative)
                    .logoLayoutId(R.id.tt_ad_logo)//logoView 建议传入GroupView类型
                    .iconImageId(R.id.iv_listitem_icon)
                    .build()
            adViewHolder.viewBinder = viewBinder
            convertView.tag = adViewHolder

            //视频广告设置播放状态回调（可选）
            ad.setTTVideoListener(object : TTVideoListener {
                override fun onVideoStart() {
                    LogUtil.d(TAG, "onVideoStart")
                }

                override fun onVideoPause() {
                    LogUtil.d(TAG, "onVideoPause")
                }

                override fun onVideoResume() {
                    LogUtil.d(TAG, "onVideoResume")
                }

                override fun onVideoCompleted() {
                    LogUtil.d(TAG, "onVideoCompleted")
                }

                override fun onVideoError(adError: AdError) {
                    LogUtil.d(TAG, "onVideoError")
                }
            })

            //绑定广告数据、设置交互回调
            bindData(convertView, adViewHolder, ad, viewBinder)
        } catch (e: Exception) {
            e.printStackTrace()
        }
        return convertView
    }

    private fun getLargeAdView(ad: TTNativeAd): View? {
        val convertView = LayoutInflater.from(MintsApplication.getContext()).inflate(R.layout.listitem_ad_large_pic, null, false)
        val adViewHolder = LargeAdViewHolder()
        adViewHolder.mTitle = convertView!!.findViewById<View>(R.id.tv_listitem_ad_title) as TextView
        adViewHolder.mDescription = convertView.findViewById<View>(R.id.tv_listitem_ad_desc) as TextView
        adViewHolder.mSource = convertView.findViewById<View>(R.id.tv_listitem_ad_source) as TextView
        adViewHolder.mLargeImage = convertView.findViewById<View>(R.id.iv_listitem_image) as ImageView
        adViewHolder.mIcon = convertView.findViewById<View>(R.id.iv_listitem_icon) as ImageView
        adViewHolder.mDislike = convertView.findViewById<View>(R.id.iv_listitem_dislike) as ImageView
        adViewHolder.mCreativeButton = convertView.findViewById<View>(R.id.btn_listitem_creative) as Button
        adViewHolder.mLogo = convertView.findViewById(R.id.tt_ad_logo) //logoView 建议传入GroupView类型
        val viewBinder: TTViewBinder = TTViewBinder.Builder(R.layout.listitem_ad_large_pic)
                .titleId(R.id.tv_listitem_ad_title)
                .decriptionTextId(R.id.tv_listitem_ad_desc)
                .sourceId(R.id.tv_listitem_ad_source)
                .mainImageId(R.id.iv_listitem_image)
                .callToActionId(R.id.btn_listitem_creative)
                .logoLayoutId(R.id.tt_ad_logo)//logoView 建议传入GroupView类型
                .iconImageId(R.id.iv_listitem_icon)
                .build()
        adViewHolder.viewBinder = viewBinder
        convertView.tag = adViewHolder
        bindData(convertView, adViewHolder, ad, viewBinder)
        if (ad.imageUrl != null) {
            adViewHolder.mLargeImage?.let { Glide.with(MintsApplication.getContext()).load(ad.imageUrl).into(it) }
        }
        return convertView
    }

    private fun getGroupAdView(ad: TTNativeAd): View? {
        val convertView = LayoutInflater.from(MintsApplication.getContext()).inflate(R.layout.listitem_ad_group_pic, null, false)
        val adViewHolder = GroupAdViewHolder()
        adViewHolder.mTitle = convertView!!.findViewById(R.id.tv_listitem_ad_title) as TextView
        adViewHolder.mSource = convertView.findViewById(R.id.tv_listitem_ad_source) as TextView
        adViewHolder.mDescription = convertView.findViewById(R.id.tv_listitem_ad_desc) as TextView
        adViewHolder.mGroupImage1 = convertView.findViewById(R.id.iv_listitem_image1) as ImageView
        adViewHolder.mGroupImage2 = convertView.findViewById(R.id.iv_listitem_image2) as ImageView
        adViewHolder.mGroupImage3 = convertView.findViewById(R.id.iv_listitem_image3) as ImageView
        adViewHolder.mIcon = convertView.findViewById(R.id.iv_listitem_icon) as ImageView
        adViewHolder.mDislike = convertView.findViewById(R.id.iv_listitem_dislike) as ImageView
        adViewHolder.mCreativeButton = convertView.findViewById(R.id.btn_listitem_creative) as Button
        adViewHolder.mLogo = convertView.findViewById(R.id.tt_ad_logo) //logoView 建议传入GroupView类型
        val viewBinder: TTViewBinder = TTViewBinder.Builder(R.layout.listitem_ad_group_pic)
                .titleId(R.id.tv_listitem_ad_title)
                .decriptionTextId(R.id.tv_listitem_ad_desc)
                .sourceId(R.id.tv_listitem_ad_source)//传第一张即可
                .mainImageId(R.id.iv_listitem_image1)
                .logoLayoutId(R.id.tt_ad_logo)//logoView 建议传入GroupView类型
                .callToActionId(R.id.btn_listitem_creative)
                .iconImageId(R.id.iv_listitem_icon)
                .build()
        adViewHolder.viewBinder = viewBinder
        convertView.tag = adViewHolder
        bindData(convertView, adViewHolder, ad, viewBinder)
        if (ad.imageList != null && ad.imageList.size >= 3) {
            val image1 = ad.imageList[0]
            val image2 = ad.imageList[1]
            val image3 = ad.imageList[2]
            if (image1 != null) {
                adViewHolder.mGroupImage1?.let { Glide.with(MintsApplication.getContext()).load(image1).into(it) }
            }
            if (image2 != null) {
                adViewHolder.mGroupImage2?.let { Glide.with(MintsApplication.getContext()).load(image2).into(it) }
            }
            if (image3 != null) {
                adViewHolder.mGroupImage3?.let { Glide.with(MintsApplication.getContext()).load(image3).into(it) }
            }
        }
        return convertView
    }

    private fun getSmallAdView(ad: TTNativeAd): View? {
        val convertView = LayoutInflater.from(MintsApplication.getContext()).inflate(R.layout.listitem_ad_small_pic, null, false)
        val adViewHolder = SmallAdViewHolder()
        adViewHolder.mTitle = convertView!!.findViewById<View>(R.id.tv_listitem_ad_title) as TextView
        adViewHolder.mSource = convertView.findViewById<View>(R.id.tv_listitem_ad_source) as TextView
        adViewHolder.mDescription = convertView.findViewById<View>(R.id.tv_listitem_ad_desc) as TextView
        adViewHolder.mSmallImage = convertView.findViewById<View>(R.id.iv_listitem_image) as ImageView
        adViewHolder.mIcon = convertView.findViewById<View>(R.id.iv_listitem_icon) as ImageView
        adViewHolder.mDislike = convertView.findViewById<View>(R.id.iv_listitem_dislike) as ImageView
        adViewHolder.mCreativeButton = convertView.findViewById<View>(R.id.btn_listitem_creative) as Button
        val viewBinder: TTViewBinder = TTViewBinder.Builder(R.layout.listitem_ad_small_pic)
                .titleId(R.id.tv_listitem_ad_title)
                .sourceId(R.id.tv_listitem_ad_source)
                .decriptionTextId(R.id.tv_listitem_ad_desc)
                .mainImageId(R.id.iv_listitem_image)
                .logoLayoutId(R.id.tt_ad_logo)//logoView 建议为GroupView 类型
                .callToActionId(R.id.btn_listitem_creative)
                .iconImageId(R.id.iv_listitem_icon)
                .build()
        adViewHolder.viewBinder = viewBinder
        convertView.tag = adViewHolder

        bindData(convertView, adViewHolder, ad, viewBinder)
        if (ad.imageUrl != null) {
            adViewHolder.mSmallImage?.let { Glide.with(MintsApplication.getContext()).load(ad.imageUrl).into(it) }
        }
        return convertView
    }

    private fun bindData(convertView: View, adViewHolder: AdViewHolder, ad: TTNativeAd, viewBinder: TTViewBinder) {
        //设置dislike弹窗，如果有
//        if (ad.hasDislike()) {
//            val ttAdDislike = ad.getDislikeDialog( MintsApplication.getContext() as Activity?)
//            adViewHolder.mDislike.setVisibility(View.VISIBLE)
//            adViewHolder.mDislike.setOnClickListener(View.OnClickListener {
//                //使用接口来展示
//                ttAdDislike.showDislikeDialog()
//                ttAdDislike.setDislikeCallback(object : TTDislikeCallback {
//                    override fun onSelected(position: Int, value: String) {}
//
//                    override fun onCancel() {}
//
//                    /**
//                     * 拒绝再次提交
//                     */
//                    /**
//                     * 拒绝再次提交
//                     */
//                    override fun onRefuse() {}
//                    override fun onShow() {}
//                })
//            })
//        } else {
//            if (adViewHolder.mDislike != null) adViewHolder.mDislike.setVisibility(View.GONE)
//        }

        //设置事件回调
        ad.setTTNativeAdListener(mTTNativeAdListener)
        //可以被点击的view, 也可以把convertView放进来意味item可被点击
        val clickViewList: MutableList<View> = ArrayList()
        clickViewList.add(convertView)
        adViewHolder.mSource?.let { clickViewList.add(it) }
        adViewHolder.mTitle?.let { clickViewList.add(it) }
        adViewHolder.mDescription?.let { clickViewList.add(it) }
        adViewHolder.mIcon?.let { clickViewList.add(it) }
        //添加点击区域
        if (adViewHolder is LargeAdViewHolder) {
            adViewHolder.mLargeImage?.let { clickViewList.add(it) }
        } else if (adViewHolder is SmallAdViewHolder) {
            adViewHolder.mSmallImage?.let { clickViewList.add(it) }
        } else if (adViewHolder is VerticalAdViewHolder) {
            adViewHolder.mVerticalImage?.let { clickViewList.add(it) }
        } else if (adViewHolder is VideoAdViewHolder) {
            adViewHolder.videoView?.let { clickViewList.add(it) }
        } else if (adViewHolder is GroupAdViewHolder) {
            adViewHolder.mGroupImage1?.let { clickViewList.add(it) }
            adViewHolder.mGroupImage2?.let { clickViewList.add(it) }
            adViewHolder.mGroupImage3?.let { clickViewList.add(it) }
        }
        //触发创意广告的view（点击下载或拨打电话）
        val creativeViewList: MutableList<View> = ArrayList()
        adViewHolder.mCreativeButton?.let { creativeViewList.add(it) }
        //重要! 这个涉及到广告计费，必须正确调用。convertView必须使用ViewGroup。
        ad.registerView((convertView as ViewGroup), clickViewList, creativeViewList, viewBinder)
        adViewHolder.mTitle?.text = ad.title //title为广告的简单信息提示
        adViewHolder.mDescription?.text = ad.description //description为广告的较长的说明
        adViewHolder.mSource?.text = if (TextUtils.isEmpty(ad.source)) "广告来源" else ad.source
        val icon = ad.iconUrl
        if (icon != null) {
            adViewHolder.mIcon?.let { Glide.with(MintsApplication.getContext()).load(icon).into(it) }
        }
        val adCreativeButton: Button? = adViewHolder.mCreativeButton
        when (ad.interactionType) {
            TTAdConstant.INTERACTION_TYPE_DOWNLOAD -> {
                adCreativeButton?.visibility = View.VISIBLE
                adCreativeButton?.text = if (TextUtils.isEmpty(ad.actionText)) "立即下载" else ad.actionText
            }
            TTAdConstant.INTERACTION_TYPE_DIAL -> {
                adCreativeButton?.visibility = View.VISIBLE
                adCreativeButton?.text = "立即拨打"
            }
            TTAdConstant.INTERACTION_TYPE_LANDING_PAGE, TTAdConstant.INTERACTION_TYPE_BROWSER -> {
                adCreativeButton?.visibility = View.VISIBLE
                adCreativeButton?.text = if (TextUtils.isEmpty(ad.actionText)) "查看详情" else ad.actionText
            }
            else -> {
                adCreativeButton?.visibility = View.GONE
            }
        }
    }

    private class VideoAdViewHolder : AdViewHolder() {
        var videoView: FrameLayout? = null
    }

    private class LargeAdViewHolder : AdViewHolder() {
        var mLargeImage: ImageView? = null
    }

    private class SmallAdViewHolder : AdViewHolder() {
        var mSmallImage: ImageView? = null
    }

    private class VerticalAdViewHolder : AdViewHolder() {
        var mVerticalImage: ImageView? = null
    }

    private class GroupAdViewHolder : AdViewHolder() {
        var mGroupImage1: ImageView? = null
        var mGroupImage2: ImageView? = null
        var mGroupImage3: ImageView? = null
    }

    private class ExpressAdViewHolder {
        var mAdContainerView: FrameLayout? = null
    }

    private open class AdViewHolder {
        var viewBinder: TTViewBinder? = null
        var mIcon: ImageView? = null
        var mDislike: ImageView? = null
        var mCreativeButton: Button? = null
        var mTitle: TextView? = null
        var mDescription: TextView? = null
        var mSource: TextView? = null
        var mLogo: RelativeLayout? = null
    }

    private var mTTNativeAdListener: TTNativeAdListener = object : TTNativeAdListener {
        override fun onAdClick() {
            LogUtil.d(TAG, "onAdClick")
        }

        override fun onAdShow() {
            LogUtil.d(TAG, "onAdShow")
        }
    }


    //渲染模板广告
//    private fun getExpressAdView(ad: TTNativeAd): View? {
//        val adViewHolder: GroMoreExpressManager.ExpressAdViewHolder
//        val convertView = LayoutInflater.from(MintsApplication.getContext()).inflate(R.layout.listitem_ad_native_express, null, false)
//        try {
//            adViewHolder = GroMoreExpressManager.ExpressAdViewHolder()
//            adViewHolder.mAdContainerView = convertView.findViewById(R.id.iv_listitem_express) as FrameLayout
//            convertView.tag = adViewHolder
//
//            //判断是否存在dislike按钮
////            if (ad.hasDislike()) {
////                ad.setDislikeCallback(MintsApplication.getContext() as Activity?, object : TTDislikeCallback {
////                    override fun onSelected(position: Int, value: String) {
////                        ToastUtil.show(MintsApplication.getContext(), "点击 $value")
////                        //用户选择不喜欢原因后，移除广告展示
////                    }
////
////                    override fun onCancel() {
////                        ToastUtil.show(MintsApplication.getContext(), "dislike 点击了取消")
////                        LogUtil.d(TAG, "dislike 点击了取消")
////                    }
////
////                    /**
////                     * 拒绝再次提交
////                     */
////                    override fun onRefuse() {}
////                    override fun onShow() {}
////                })
////            }
//
//            //设置点击展示回调监听
//            ad.setTTNativeAdListener(object : TTNativeExpressAdListener {
//                override fun onAdClick() {
//                    LogUtil.d(TAG, "onAdClick")
//                    ToastUtil.show(MintsApplication.getContext(), "模板广告被点击")
//                }
//
//                override fun onAdShow() {
//                    LogUtil.d(TAG, "onAdShow")
//                    ToastUtil.show(MintsApplication.getContext(), "模板广告show")
//                }
//
//                override fun onRenderFail(view: View, msg: String, code: Int) {
//                    ToastUtil.show(MintsApplication.getContext(), "模板广告渲染失败code=$code,msg=$msg")
//                    LogUtil.d(TAG, "onRenderFail   code=$code,msg=$msg")
//                }
//
//                // ** 注意点 ** 不要在广告加载成功回调里进行广告view展示，要在onRenderSucces进行广告view展示，否则会导致广告无法展示。
//                // 另外，不要直接使用参数view，而是通过ad.getExpressView()来获取广告view。
//                override fun onRenderSuccess(view: View, width: Float, height: Float) {
//                    LogUtil.d(TAG, "onRenderSuccess")
//                    ToastUtil.show(MintsApplication.getContext(), "模板广告渲染成功:width=$width,height=$height")
//                    //回调渲染成功后将模板布局添加的父View中
//                    if (adViewHolder.mAdContainerView != null) {
//                        //获取视频播放view,该view SDK内部渲染，在媒体平台可配置视频是否自动播放等设置。
//                        val sWidth: Int
//                        val sHeight: Int
//                        val video = ad.expressView // 使用该方式获取广告view，而不是直接使用onRenderSuccess的参数view。
//                        if (width == TTAdSize.FULL_WIDTH.toFloat() && height == TTAdSize.AUTO_HEIGHT.toFloat()) {
//                            sWidth = FrameLayout.LayoutParams.MATCH_PARENT
//                            sHeight = FrameLayout.LayoutParams.WRAP_CONTENT
//                        } else {
//                            sWidth = UIUtils.getScreenWidthDp(MintsApplication.getContext()).toInt()
//                            sHeight = (sWidth * height / width).toInt()
//                        }
//                        if (video != null) {
//                            if (video.parent == null) {
//                                val layoutParams = FrameLayout.LayoutParams(sWidth, sHeight)
//                                adViewHolder.mAdContainerView!!.removeAllViews()
//                                adViewHolder.mAdContainerView!!.addView(video, layoutParams)
//                            }
//                        }
//                    }
//                }
//            })
//
//
//            //视频广告设置播放状态回调（可选）
//            ad.setTTVideoListener(object : TTVideoListener {
//                override fun onVideoStart() {
//                    ToastUtil.show(MintsApplication.getContext(), "模板广告视频开始播放")
//                    LogUtil.d(TAG, "onVideoStart")
//                }
//
//                override fun onVideoPause() {
//                    ToastUtil.show(MintsApplication.getContext(), "模板广告视频暂停")
//                    LogUtil.d(TAG, "onVideoPause")
//                }
//
//                override fun onVideoResume() {
//                    ToastUtil.show(MintsApplication.getContext(), "模板广告视频继续播放")
//                    LogUtil.d(TAG, "onVideoResume")
//                }
//
//                override fun onVideoCompleted() {
//                    ToastUtil.show(MintsApplication.getContext(), "模板播放完成")
//                    LogUtil.d(TAG, "onVideoCompleted")
//                }
//
//                override fun onVideoError(adError: AdError) {
//                    ToastUtil.show(MintsApplication.getContext(), "模板广告视频播放出错")
//                    LogUtil.d(TAG, "onVideoError")
//                }
//            })
//            ad.render()
//        } catch (e: Exception) {
//            e.printStackTrace()
//        }
//        return convertView
//    }

}